/**
 * Copyright (c) 2009 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
* @brief Example template project.
* @defgroup nrf_templates_example Example Template
*
*/

#include <stdbool.h>
#include <stdint.h>

#include "nrf.h"
#include "nordic_common.h"
#include "boards.h"
#include "nrf_drv_gpiote.h"

  //global gpiote input config variable which is reachable within the gpiote event handler. Alternatively you could create a config within the handler
  nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_HITOLO(false); 
  //boolean flags to monitor wether or not the sense polarity for a given input is flipped.
  bool pol0 = false;  
  bool pol1 = false;  
  bool pol2 = false;  
  bool pol3 = false;  
  
static void gpiote_evt_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
 
  switch(pin)
  {
    case BSP_BUTTON_0:
    {
      //Disable the other input events.
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_1);
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_2);
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_3);

      if(pol0) //if polarity is flipped
      {
        nrf_drv_gpiote_in_uninit(BSP_BUTTON_0);                                                 //uninit gpiote pin
        in_config.sense = NRF_GPIOTE_POLARITY_HITOLO;                                           //change sense polarity for the pin
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(BSP_BUTTON_0, &in_config, gpiote_evt_handler));  //re-init the pin
        nrf_drv_gpiote_in_event_enable(BSP_BUTTON_0, true);                                     //enable event
        pol0=false;                                                                             //change polarity flag
        nrf_drv_gpiote_out_toggle(BSP_LED_0);                                                   //toggle LED
      }else //if polarity isn't flipped
      {
        nrf_drv_gpiote_in_uninit(BSP_BUTTON_0);
        in_config.sense = NRF_GPIOTE_POLARITY_LOTOHI;
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(BSP_BUTTON_0, &in_config, gpiote_evt_handler));
        nrf_drv_gpiote_in_event_enable(BSP_BUTTON_0, true);
        pol0=true;
        nrf_drv_gpiote_out_toggle(BSP_LED_0);
      }
      
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_1, true);
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_2, true);
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_3, true);
      break;
    }
    case BSP_BUTTON_1:
    {
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_0);
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_2);
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_3);
      if(pol1)
      {
        nrf_drv_gpiote_in_uninit(BSP_BUTTON_1);
        in_config.sense = NRF_GPIOTE_POLARITY_HITOLO;
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(BSP_BUTTON_1, &in_config, gpiote_evt_handler));
        nrf_drv_gpiote_in_event_enable(BSP_BUTTON_1, true);
        pol1=false;
        nrf_drv_gpiote_out_toggle(BSP_LED_1);
      }else
      {
        nrf_drv_gpiote_in_uninit(BSP_BUTTON_1);
        in_config.sense = NRF_GPIOTE_POLARITY_LOTOHI;
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(BSP_BUTTON_1, &in_config, gpiote_evt_handler));
        nrf_drv_gpiote_in_event_enable(BSP_BUTTON_1, true);
        pol1=true;
        nrf_drv_gpiote_out_toggle(BSP_LED_1);
      }
      
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_0, true);
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_2, true);
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_3, true);
      break;
    }
    case BSP_BUTTON_2:
    {
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_0);
      if(pol2)
      {
        nrf_drv_gpiote_in_uninit(BSP_BUTTON_2);
        in_config.sense = NRF_GPIOTE_POLARITY_HITOLO;
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(BSP_BUTTON_2, &in_config, gpiote_evt_handler));
        nrf_drv_gpiote_in_event_enable(BSP_BUTTON_2, true);
        pol2=false;
        nrf_drv_gpiote_out_toggle(BSP_LED_2);
      }else
      {
        nrf_drv_gpiote_in_uninit(BSP_BUTTON_2);
        in_config.sense = NRF_GPIOTE_POLARITY_LOTOHI;
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(BSP_BUTTON_2, &in_config, gpiote_evt_handler));
        nrf_drv_gpiote_in_event_enable(BSP_BUTTON_2, true);
        pol2=true;
        nrf_drv_gpiote_out_toggle(BSP_LED_2);
      }
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_0, true);

      break;
    }
    case BSP_BUTTON_3:
    {
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_0);
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_1);
      nrf_drv_gpiote_in_event_disable(BSP_BUTTON_2);
      if(pol3)
      {
        nrf_drv_gpiote_in_uninit(BSP_BUTTON_3);
        in_config.sense = NRF_GPIOTE_POLARITY_HITOLO;
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(BSP_BUTTON_3, &in_config, gpiote_evt_handler));
        nrf_drv_gpiote_in_event_enable(BSP_BUTTON_3, true);
        pol3=false;
        nrf_drv_gpiote_out_toggle(BSP_LED_3);
      }else
      {
        nrf_drv_gpiote_in_uninit(BSP_BUTTON_3);
        in_config.sense = NRF_GPIOTE_POLARITY_LOTOHI;
        APP_ERROR_CHECK(nrf_drv_gpiote_in_init(BSP_BUTTON_3, &in_config, gpiote_evt_handler));
        nrf_drv_gpiote_in_event_enable(BSP_BUTTON_3, true);
        pol3=true;
        nrf_drv_gpiote_out_toggle(BSP_LED_3);
      }
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_0, true);
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_1, true);
      nrf_drv_gpiote_in_event_enable(BSP_BUTTON_2, true);
      break;
    }
    
  }
  
}

static void gpiote_init(void)
{
  ret_code_t err_code;
  err_code = nrf_drv_gpiote_init();
  APP_ERROR_CHECK(err_code);
  
  in_config.pull = NRF_GPIO_PIN_PULLUP;
  
  //Config pca10040 buttons as inputs, triggering on a high to low transition.
  
  err_code = nrf_drv_gpiote_in_init(BSP_BUTTON_0, &in_config, gpiote_evt_handler);
  err_code = nrf_drv_gpiote_in_init(BSP_BUTTON_1, &in_config, gpiote_evt_handler);
  err_code = nrf_drv_gpiote_in_init(BSP_BUTTON_2, &in_config, gpiote_evt_handler);
  err_code = nrf_drv_gpiote_in_init(BSP_BUTTON_3, &in_config, gpiote_evt_handler);
  nrf_drv_gpiote_in_event_enable(BSP_BUTTON_0, true);
  nrf_drv_gpiote_in_event_enable(BSP_BUTTON_1, true);
  nrf_drv_gpiote_in_event_enable(BSP_BUTTON_2, true);
  nrf_drv_gpiote_in_event_enable(BSP_BUTTON_3, true);
  
  //config LEDS as outputs
  nrf_drv_gpiote_out_config_t out_config = GPIOTE_CONFIG_OUT_SIMPLE(false);
  err_code = nrf_drv_gpiote_out_init(BSP_LED_0, &out_config);
  APP_ERROR_CHECK(err_code);
  err_code = nrf_drv_gpiote_out_init(BSP_LED_1, &out_config);
  APP_ERROR_CHECK(err_code);
  err_code = nrf_drv_gpiote_out_init(BSP_LED_2, &out_config);
  APP_ERROR_CHECK(err_code);
  err_code = nrf_drv_gpiote_out_init(BSP_LED_3, &out_config);
  APP_ERROR_CHECK(err_code);
}

/**
 * @brief Function for application main entry.
 */
int main(void)
{

  gpiote_init();
  
  
    while (true)
    {
        // Do nothing.
    }
}
/** @} */
